// ****************************************************************************
// 
// UnpackCL
// Copyright (C) 2005-2006  Moitah (moitah@yahoo.com)
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// ****************************************************************************

using System;
using System.IO;

namespace JDP {
	class UnpackCL {
		private static bool _loading;

		[STAThread]
		static void Main(string[] args) {
			bool badArgs = false;
			int argCount = args.Length;
			int argIndex;
			string pathSrc = null;
			string pathDst = null;
			bool showInfo = false;

			Console.WriteLine("UnpackCL v1.2.1 by Moitah");
			Console.WriteLine("Based on MPEG4 Modifier v1.4.2");
			Console.WriteLine("http://www.moitah.net/");
			Console.WriteLine();

			if (argCount < 1) {
				badArgs = true;
			}
			else {
				for (argIndex = 0; argIndex < argCount; argIndex++) {
					if (args[argIndex] == "-i") {
						showInfo = true;
					}
					else {
						break;
					}
				}
				if ((argIndex >= (argCount - 2)) && (argIndex < argCount)) {
					pathSrc = args[argIndex++];
					if (argIndex < argCount) {
						pathDst = args[argIndex++];
					}
				}
				else {
					badArgs = true;
				}
			}
			if (badArgs) {
				Console.WriteLine("Arguments: [switches] source_path [dest_path]");
				Console.WriteLine("Switches:");
				Console.WriteLine("    -i    Displays detailed information about the video");
				return;
			}

			try {
				Unpack(pathSrc, pathDst, showInfo);
			}
			catch (Exception ex) {
				Console.WriteLine("Error: " + ex.Message);
				return;
			}
		}

		private static void Unpack(string pathSrc, string pathDst, bool showInfo) {
			if (!File.Exists(pathSrc)) {
				throw new Exception("Source file doesn't exist");
			}
			if ((pathDst != null) && File.Exists(pathDst)) {
				throw new Exception("Dest file already exists");
			}

			_loading = true;
			AVIModifier aviMod = new AVIModifier(pathSrc);
			MPEG4FrameModifier mp4Mod = new MPEG4FrameModifier();
			aviMod.FrameModifier = mp4Mod;
			mp4Mod.VideoModifier = aviMod;
			aviMod.ProgressCallback = new ProgressCallback(Progress);
			aviMod.Preview();
			Progress(1.0);
			Console.WriteLine();

			if (showInfo) {
				Console.Write("---------- Video Info ----------{1}{0}--------------------------------{1}",
					mp4Mod.GenerateStats(), Environment.NewLine);
			}

			if (pathDst == null) {
				if (!showInfo) {
					Console.WriteLine("Packed bitstream: {0}", mp4Mod.IsPacked ? "Yes" : "No");
				}
				return;
			}

			if (!mp4Mod.IsPacked) {
				throw new Exception("Video isn't packed");
			}

			mp4Mod.NewIsPacked = false;
			mp4Mod.UserDataList = mp4Mod.SuggestedUserData();

			_loading = false;
			aviMod.Write(pathDst);
			Progress(1.0);
			Console.WriteLine();

			Console.WriteLine("Successfully unpacked video");
		}

		private static bool Progress(double progress) {
			if (progress < 0.0) {
				progress = 0.0;
			}
			else if (progress > 1.0) {
				progress = 1.0;
			}

			Console.Write("\r{0}: {1:0.0}%", (_loading ? "Loading" : "Saving"), progress * 100.0);

			return false;
		}
	}
}